//
//  AgoraRtmKit.h
//  AgoraRtmKit
//
//  Copyright (c) 2019 Agora.io. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "AgoraRtmCallKit.h"

/**
 The `AgoraRtmKit` class is the entry point of Agora's RTM SDK.
 */
@class AgoraRtmKit;
/**
 The AgoraRtmMessage class provides Agora RTM message methods that can be invoked by your app.
 */
@class AgoraRtmMessage;
/**
 The AgoraRtmChannel class provides Agora RTM channel methods that can be invoked by your app.
 */
@class AgoraRtmChannel;
/**
 The AgoraRtmCallKit class provides Agora RTM call methods that can be invoked by your app.
 */
@class AgoraRtmCallKit;

@class AgoraRtmPeerOnlineStatus;

/** Message types.
 */
typedef NS_ENUM(NSInteger, AgoraRtmMessageType) {
  /** 0: Undefined message type. */
  AgoraRtmMessageTypeUndefined = 0,
  /** 1: A text message. */
  AgoraRtmMessageTypeText = 1,
};

/** Error codes of sending a peer-to-peer message. */
typedef NS_ENUM(NSInteger, AgoraRtmSendPeerMessageErrorCode) {
    /** 0: The user has successfully sent the peer-to-peer message. */
    AgoraRtmSendPeerMessageErrorOk = 0,
    /** 1: The user fails to send the peer-to-peer message. */
    AgoraRtmSendPeerMessageErrorFailure = 1,
    /** 2: A timeout occurs when sending the peer-to-peer message. The current timeout is set as 5 seconds. */
    AgoraRtmSendPeerMessageErrorTimeout = 2,
    /** 3: The user is offline and has not received the peer-to-peer message. */
    AgoraRtmSendPeerMessageErrorPeerUnreachable = 3,
};

/** Connection states between the SDK and the Agora RTM system. */
typedef NS_ENUM(NSInteger, AgoraRtmConnectionState) {
/** 1: The initial state. The SDK is not connected to the Agora RTM system.
<p>When the app calls the [loginByToken]([AgoraRtmKit loginByToken:user:completion:]) method, the SDK starts to log in the Agora RTM system, triggers the [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callback, and switches to the `AgoraRtmConnectionStateConnecting` state.
*/
    AgoraRtmConnectionStateDisconnected = 1,
/** 2: The SDK is logging in the Agora RTM system.
<p><li>Success: The SDK triggers the [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callback and switches to the `AgoraRtmConnectionStateConnected` state.
<li>Failure: The SDK triggers the [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callback and switches to the `AgoraRtmConnectionStateDisConnected` state.
*/
    AgoraRtmConnectionStateConnecting = 2,
/** 3: The SDK has logged in the Agora RTM system.
<p><li>If the connection between the SDK and the Agora RTM system is interrupted because of network issues, the SDK triggers the [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callback and switches to the `AgoraRtmConnectionStateReConnecting` state.
<li>If the login is banned by the server, for example, another instance logs in with the same user ID from a different device, the SDK triggers the [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callback and switches to the `AgoraRtmConnectionStateAborted` state.
<li>If the app calls the [logoutWithCompletion]([AgoraRtmKit logoutWithCompletion:]) method and the SDK successfully logs out of the Agora RTM system, the SDK triggers the [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callback and switches to the `AgoraRtmConnectionStateDisConnected` state.
*/
    AgoraRtmConnectionStateConnected = 3,
/** 4: The SDK keeps re-logging in the Agora RTM system.
<p><li>If the SDK successfully re-logs in the Agora RTM system, it triggers the [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callback and switches to the `AgoraRtmConnectionStateConnected` state.
<li>If the SDK fails to re-log in the Agora RTM system, the SDK stays in the `AgoraRtmConnectionStateReConnecting` state and keeps logging in.
<li>If the login is rejected by the server, the SDK triggers the [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callback and switches to the `AgoraRtmConnectionStateAborted` state.
*/
    AgoraRtmConnectionStateReconnecting = 4,
/** 5: The SDK gives up logging in the Agora RTM system, possibly because another instance has logged in the Agora RTM system with the same user ID.
<p>Call the [logoutWithCompletion]([AgoraRtmKit logoutWithCompletion:]) method before calling the [loginByToken]([AgoraRtmKit loginByToken:user:completion:]) method to log in the Agora RTM system again.
*/
    AgoraRtmConnectionStateAborted = 5,
};

/** Reasons for a connection state change. */
typedef NS_ENUM(NSInteger, AgoraRtmConnectionChangeReason) {
    /** 1: The SDK is logging in the Agora RTM system. */
    AgoraRtmConnectionChangeReasonLogin = 1,
    /** 2: The SDK has logged in the Agora RTM system. */
    AgoraRtmConnectionChangeReasonLoginSuccess = 2,
    /** 3: The SDK fails to log in the Agora RTM system, because, for example, the token expires.*/
    AgoraRtmConnectionChangeReasonLoginFailure = 3,
    /** 4: The login has timed out for 10 seconds, and the SDK stops logging in. */
    AgoraRtmConnectionChangeReasonLoginTimeout = 4,
    /** 5: The connection between the SDK and the Agora RTM system has been interrupted for more than four seconds. */
    AgoraRtmConnectionChangeReasonInterrupted = 5,
    /** 6: The SDK has logged out of the Agora RTM system. */
    AgoraRtmConnectionChangeReasonLogout = 6,
    /** 7: Login is banned by the Agora RTM server. */
    AgoraRtmConnectionChangeReasonBannedByServer = 7,
    /** 8: Another instance has logged in the Agora RTM system with the same user ID. */
    AgoraRtmConnectionChangeReasonRemoteLogin = 8,
};

/** Login error codes. */
typedef NS_ENUM(NSInteger, AgoraRtmLoginErrorCode) {
    /** 0: Login succeeds. No error occurs. */
    AgoraRtmLoginErrorOk = 0,
    /** 1: Login fails for reasons unknown.*/
    AgoraRtmLoginErrorUnknown = 1,
    /** 2: Login is rejected, possibly because the SDK is not initialized or is rejected by the server. */
    AgoraRtmLoginErrorRejected = 2,
    /** 3: Invalid login arguments.*/
    AgoraRtmLoginErrorInvalidArgument = 3,
    /** 4: Invalid App ID. */
    AgoraRtmLoginErrorInvalidAppId = 4,
    /** 5: Invalid token. */
    AgoraRtmLoginErrorInvalidToken = 5,
    /** 6: The token has expired and login is rejected.*/
    AgoraRtmLoginErrorTokenExpired = 6,
    /** 7: Unauthorized login. */
    AgoraRtmLoginErrorNotAuthorized = 7,
    /** 8: The SDK is either logging in or has logged in the Agora RTM system. */
    AgoraRtmLoginErrorAlreadyLogin = 8,
    /** 9: A login timeout. The current timeout is set as 10 seconds.*/
    AgoraRtmLoginErrorTimeout = 9,
    /** 10: Login or logout too often.  */
    AgoraRtmLoginErrorLoginTooOften = 10,
};

/** Logout error codes. */
typedef NS_ENUM(NSInteger, AgoraRtmLogoutErrorCode) {
  /** 0: Logout succeeds. */
  AgoraRtmLogoutErrorOk = 0,
  /** 1: Logout fails. Maybe the SDK is not initialized or the user has not logged in. */
  AgoraRtmLogoutErrorRejected = 1,
};
/** AgoraRtmQueryPeersOnline error codes. */
typedef NS_ENUM(NSInteger, AgoraRtmQueryPeersOnlineErrorCode) {
/** The method call succeeds.*/
  AgoraRtmQueryPeersOnlineErrorOk = 0,
/** The method call fails.*/
  AgoraRtmQueryPeersOnlineErrorFailure = 1,
/** The method call fails. The argument is invalid. */
  AgoraRtmQueryPeersOnlineErrorInvalidArgument = 2,
/** You are not logged in. */
  AgoraRtmQueryPeersOnlineErrorRejected = 3,
/** Has not received a response from the server for 10 seconds. */
  AgoraRtmQueryPeersOnlineErrorTimeout = 4,
/** Call frequency of this method exceeds the limit of 10 queries every 5 seconds. */
  AgoraRtmQueryPeersOnlineErrorTooOften = 5,
};

typedef NS_ENUM(NSInteger, AgoraRtmRenewTokenErrorCode) {
/** The method call succeeds. */
  AgoraRtmRenewTokenErrorOk = 0,
  /** The method call fails. */
  AgoraRtmRenewTokenErrorFailure = 1,
  /** The argument is invalid */
  AgoraRtmRenewTokenErrorInvalidArgument = 2,
  /** The RTM service is not initialized. You are rejected. */
  AgoraRtmRenewTokenErrorRejected = 3,
  /** The call frequency of this method exceeds the limit of 2 qureis per second. */
  AgoraRtmRenewTokenErrorTooOften = 4,
  /** The token you put has expired. */
  AgoraRtmRenewTokenErrorTokenExpired = 5,
  /** The token you put is invalid. */
  AgoraRtmRenewTokenErrorInvalidToken = 6,
};


/** Occurs after the app calls the [loginByToken]([AgoraRtmKit loginByToken:user:completion:]) method.

 @param errorCode Login error code. See AgoraRtmLoginErrorCode.
 */
typedef void (^AgoraRtmLoginBlock)(AgoraRtmLoginErrorCode errorCode);
/** Indicates the results of calling the [logoutWithCompletion]([AgoraRtmKit logoutWithCompletion:]) method.

 @param errorCode Logout error code. See AgoraRtmLogoutErrorCode.
 */
typedef void (^AgoraRtmLogoutBlock)(AgoraRtmLogoutErrorCode errorCode);
/** Indicates the results of calling the [sendMessage]([AgoraRtmKit sendMessage:toPeer:completion:]) method.

 @param errorCode Error code of sending the peer-to-peer message. See AgoraRtmSendPeerMessageErrorCode.
 */
typedef void (^AgoraRtmSendPeerMessageBlock)(AgoraRtmSendPeerMessageErrorCode errorCode);
/** Indicates the results of calling the [queryPeersOnlineStatus]([AgoraRtmKit queryPeersOnlineStatus:completion:]) method. 

@param peerOnlineStatus The online status list of the users. 
@errorCode See AgoraRtmQueryPeersOnlineErrorCode. 
*/
typedef void (^AgoraRtmQueryPeersOnlineBlock)(NSArray<AgoraRtmPeerOnlineStatus *> * peerOnlineStatus, AgoraRtmQueryPeersOnlineErrorCode errorCode);
/** Indicates the result of calling the [renewToken]([AgoraRtmKit renewToken:completion:]) method. 
@param token The new Token. 
@errorCode See AgoraRtmRenewTokenErrorCode.
*/
typedef void (^AgoraRtmRenewTokenBlock)(NSString *token, AgoraRtmRenewTokenErrorCode errorCode);

/** Agora RTM message methods.
 */
__attribute__((visibility("default"))) @interface AgoraRtmMessage: NSObject
/** Agora RTM message type. Text messages only.
 */
@property (nonatomic, assign, readonly) AgoraRtmMessageType type;
/** Agora RTM message content.
 */
@property (nonatomic, copy, nonnull) NSString *text;

/** Creates a text message to be sent.

 @param text Text message with a string of less than 32 KB.

 @return An AgoraRtmMessage instance.

 */
- (instancetype _Nonnull)initWithText:(NSString * _Nonnull)text;
@end

__attribute__((visibility("default"))) @interface AgoraRtmPeerOnlineStatus: NSObject
/** The user ID of the specified user. */
@property (nonatomic, copy, nonnull) NSString *peerId;
/** Whether the user is online or not. 

- true: Online.
- false: Offline.
*/
@property (nonatomic, assign) BOOL isOnline;
@end

/** The AgoraRtmChannelDelegate protocol enables Agora RTM channel callback event notifications to your app.
 */
@protocol AgoraRtmChannelDelegate;
/** The AgoraRtmCallDelegate protocol enables Agora RTM call callback event notifications to your app.
 */
@protocol AgoraRtmCallDelegate ;

/** The AgoraRtmDelegate protocol enables Agora RTM callback event notifications to your app.
 */
@protocol AgoraRtmDelegate <NSObject>
@optional

/** Occurs when the connection state between the SDK and the Agora RTM system changes.

 @param kit An [AgoraRtmKit](AgoraRtmKit) instance.
 @param state The new connection state. See AgoraRtmConnectionState.
 @param reason The reason for the connection state change. See AgoraRtmConnectionChangeReason.

 */
- (void)rtmKit:(AgoraRtmKit * _Nonnull)kit connectionStateChanged:(AgoraRtmConnectionState)state reason:(AgoraRtmConnectionChangeReason)reason;
/** Occurs when the local user receives a peer-to-peer message.

 @param kit An [AgoraRtmKit](AgoraRtmKit) instance.
 @param message The received message.
 @param peerId The user ID of the sender.

 */
- (void)rtmKit:(AgoraRtmKit * _Nonnull)kit messageReceived:(AgoraRtmMessage * _Nonnull)message fromPeer:(NSString * _Nonnull)peerId;

/** Occurs when your token expires. 

@param kit An AgoraRtmKit instance.
*/
- (void)rtmKitTokenDidExpire:(AgoraRtmKit * _Nonnull)kit;
@end


/** The entry point to the Agora RTM system.
 */
__attribute__((visibility("default"))) @interface AgoraRtmKit: NSObject
/**-----------------------------------------------------------------------------
 * @name Core Methods
 * -----------------------------------------------------------------------------
 */
/** AgoraRtmDelegate enables Agora RTM callback event notifications to your app.
 */
@property (atomic, weak, nullable) id<AgoraRtmDelegate> agoraRtmDelegate;
/** **DEPRECATED** The property for managing channels for the local user.
 */
@property (atomic, readonly, nullable) NSMutableDictionary<NSString *, AgoraRtmChannel *> *channels __deprecated;

@property (atomic, strong, nullable) AgoraRtmCallKit *rtmCallKit;

/** Initializes an `AgoraRtmKit` instance.

 The Agora RTM SDK supports multiple `AgoraRtmKit` instances. All methods in the `AgoraRtmKit` class, except the [destroyChannelWithId](destroyChannelWithId:) method, are executed asynchronously.

 @param appId The App ID issued to you from the Agora Dashboard. Apply for a new App ID from Agora if it is missing from your kit.

 @param delegate AgoraRtmDelegate invokes callbacks to be passed to the app on Agora RTM SDK runtime events.

 @return The SDK returns:

 - An `AgoraRtmKit` instance if this method call succeeds.
 - `nil` if this method call fails for the reason that the length of the `appId` is not 32 characters.

 */
- (instancetype _Nullable)initWithAppId:(NSString * _Nonnull)appId
                              delegate:(id <AgoraRtmDelegate> _Nullable)delegate;

/** Allows a user to log in the Agora RTM system.
 
 **Note:** If you log in with the same user ID from a different instance, you will be kicked out of your previous login and removed from previously joined channels.

 After the app calls this method, the local user receives the [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callback and switches to the `AgoraRtmConnectionStateConnecting` state.

 - Success: The local user receives the [AgoraRtmLoginBlock](AgoraRtmLoginBlock) and [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callbacks, and switches to the `AgoraRtmConnectionStateConnected` state.
 - Failure: The local user receives the [AgoraRtmLoginBlock](AgoraRtmLoginBlock) and [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callbacks, and switches to the `AgoraRtmConnectionStateDisConnected` state.

 @param token A token generated by the app server and used to log in the Agora RTM system. `token` is used when dynamic authentication is enabled. Set `token` as `nil` at the integration and test stages.

 @param userId The user ID of the user logging in the Agora RTM system. The string length must be less than 64 bytes with the following character scope:

 - The 26 lowercase English letters: a to z
 - The 26 uppercase English letters: A to Z
 - The 10 numbers: 0 to 9
 - Space
 - "!", "#", "$", "%", "&", "(", ")", "+", "-", ":", ";", "<", "=", ".", ">", "?", "@", "[", "]", "^", "_", " {", "}", "|", "~", ","

 **Note:** Do not set `userId` as nil and do not start with a space.

 @param completionBlock [AgoraRtmLoginBlock](AgoraRtmLoginBlock), which indicates the login result. See AgoraRtmLoginErrorCode for the error codes.

 */
- (void)loginByToken:(NSString * _Nullable)token
                user:(NSString * _Nonnull)userId
          completion:(AgoraRtmLoginBlock _Nullable)completionBlock;

/** Allows a user to log out of the Agora RTM system.

 - Success: The local user receives the [AgoraRtmLogoutBlock](AgoraRtmLogoutBlock) and [connectionStateChanged]([AgoraRtmDelegate rtmKit:connectionStateChanged:reason:]) callbacks, and switches to the `AgoraRtmConnectionStateDisConnected` state.
 - Failure: The local user receives the [AgoraRtmLogoutBlock](AgoraRtmLogoutBlock) callback.

 @param completionBlock [AgoraRtmLogoutBlock](AgoraRtmLogoutBlock), which indicates the logout result. See AgoraRtmLogoutErrorCode for the error codes.
 */
- (void)logoutWithCompletion:(AgoraRtmLogoutBlock _Nullable )completionBlock;
/**
Renews the token.

@param token Your new token. 
@param completionBlock An AgoraRtmRenewTokenBlock instance.
*/
- (void)renewToken:(NSString * _Nonnull)token completion:(AgoraRtmRenewTokenBlock _Nullable)completionBlock;

/** Allows a user to send a peer-to-peer message to a specific peer user.
 
 **Note:** You can send messages, including peer-to-peer and channel messages, at a maximum speed of 60 queries per second.

 - Success:
    - The local user receives the [AgoraRtmSendPeerMessageBlock](AgoraRtmSendPeerMessageBlock) callback.
    - The specified remote user receives the [messageReceived]([AgoraRtmDelegate rtmKit:messageReceived:fromPeer:]) callback.
 - Failure: The local user receives the [AgoraRtmSendPeerMessageBlock](AgoraRtmSendPeerMessageBlock) callback with an error. See AgoraRtmSendPeerMessageErrorCode for the error codes.

 @param message The message to be sent. For information about creating a message, see AgoraRtmMessage.

 @param peerId The user ID of the remote user.

 @param completionBlock [AgoraRtmSendPeerMessageBlock](AgoraRtmSendPeerMessageBlock), which indicates the result of sending a peer-to-peer message. See AgoraRtmSendPeerMessageErrorCode for the error codes.
 */
- (void)sendMessage:(AgoraRtmMessage * _Nonnull)message
             toPeer:(NSString * _Nonnull)peerId
         completion:(AgoraRtmSendPeerMessageBlock _Nullable)completionBlock;

/** Creates an Agora RTM channel.

 **Note:** You can create a maximum of 20 `AgoraRtmChannel` instances in an `AgoraRtmKit` instance at the same time.

 @param channelId The unique channel name of the Agora RTM session. The string length must not exceed 64 bytes with the following character scope:
 
 - The 26 lowercase English letters: a to z
 - The 26 uppercase English letters: A to Z
 - The 10 numbers: 0 to 9
 - Space
 - "!", "#", "$", "%", "&", "(", ")", "+", "-", ":", ";", "<", "=", ".", ">", "?", "@", "[", "]", "^", "_", " {", "}", "|", "~", ","

 **Note:** `channelId` cannot be empty or set as nil.

 @param delegate [AgoraRtmChannelDelegate](AgoraRtmChannelDelegate) invokes callbacks to be passed to the app on Agora RTM SDK runtime events.

 @return The SDK returns:

 - An [AgoraRtmChannel](AgoraRtmChannel) instance if this method call succeeds.
 - `nil` if this method call fails for reasons that the `channelId` is invalid, an existing channel with the same `channelId` exists, or the number of channels is over the limit.

*/
- (AgoraRtmChannel * _Nullable)createChannelWithId:(NSString * _Nonnull)channelId
                                    delegate:(id <AgoraRtmChannelDelegate> _Nullable)delegate;
/** Destroys an AgoraRtmChannel instance.
 
 **Note:** Do not call this method in any of your callbacks.

 @param channelId The channel ID of the channel to be destroyed.
*/
- (BOOL)destroyChannelWithId: (NSString * _Nonnull) channelId;

/** Gets the AgoraRtmCallKit instance.

 @return The AgoraRtmCallKit instance.
 */
- (AgoraRtmCallKit * _Nullable)getRtmCallKit;
/** Queries the online status of the specified user(s).

@param peerIds User IDs of the specified users.
@param completionBlock An AgoraRtmQueryPeersOnlineBlock instance. 
*/
- (void)queryPeersOnlineStatus: (NSArray<NSString*> * _Nonnull) peerIds completion:(AgoraRtmQueryPeersOnlineBlock _Nullable)completionBlock;

/**-----------------------------------------------------------------------------
 * @name Customized Methods (Technical Preview)
 * -----------------------------------------------------------------------------
 */

/** Provides the technical preview functionalities or special customizations by configuring the SDK with JSON options.

 **Note:**

 The JSON options are not public by default. Agora is working on making commonly used JSON options public in a standard way. Contact [support@agora.io](mailto:support@agora.io) for more information.

 @param parameters SDK options in JSON format.
 */
- (int)setParameters:(NSString * _Nonnull)parameters;

@end


#pragma mark channel

/** Error codes of sending a channel message. */
typedef NS_ENUM(NSInteger, AgoraRtmSendChannelMessageErrorCode) {
  /** 0: The user has successfully sent the channel message. */
  AgoraRtmSendChannelMessageErrorOk = 0,
  /** 1: The user fails to send the channel message. */
  AgoraRtmSendChannelMessageErrorFailure = 1,
  /** 2: A timeout occurs when sending the channel message. The current timeout is set as five seconds. */
  AgoraRtmSendChannelMessageErrorTimeout = 2,
};

/** Error codes of a user joining a channel.
*/
typedef NS_ENUM(NSInteger, AgoraRtmJoinChannelErrorCode) {
    /** 0: The user joins the channel successfully. */
    AgoraRtmJoinChannelErrorOk = 0,
    /** 1: The user fails to join the channel. */
    AgoraRtmJoinChannelErrorFailure = 1,
    /** 2: The user cannot join the channel, possibly because the user has already joined the channel. */
    AgoraRtmJoinChannelErrorRejected = 2,
    /** 3: The user fails to join the channel because the argument is invalid. */
    AgoraRtmJoinChannelErrorInvalidArgument = 3,
    /** 4: A timeout occurs when joining the channel. The current timeout is set as five seconds.  */
    AgoraRtmJoinChannelErrorTimeout = 4,
};

/** Error codes of a user leaving a channel.
*/
typedef NS_ENUM(NSInteger, AgoraRtmLeaveChannelErrorCode) {
    /** 0: The user leaves the channel successfully. */
    AgoraRtmLeaveChannelErrorOk = 0,
    /** 1: The user fails to leave the channel. */
    AgoraRtmLeaveChannelErrorFailure = 1,
    /** 2: The user cannot leave the channel, possibly because the user is not in the channel. */
    AgoraRtmLeaveChannelErrorRejected = 2,
};

/** Error codes of retrieving the member list of a channel.
*/
typedef NS_ENUM(NSInteger, AgoraRtmGetMembersErrorCode) {
    /** 0: The user retrieves the member list of a channel successfully. */
    AgoraRtmGetMembersErrorOk = 0,
    /** 1: The user fails to retrieve the member list of a channel. */
    AgoraRtmGetMembersErrorFailure = 1,
    /** 2: The user cannot retrieve the member list of a channel. */
    AgoraRtmGetMembersErrorRejected = 2,
    /** 3: A timeout occurs when retreiving the member list of a channel. The current timeout is set as 5 seconds.*/
    AgoraRtmGetMembersErrorTimeout = 3,
};

/** Agora RTM channel member methods.
 */
__attribute__((visibility("default"))) @interface AgoraRtmMember: NSObject
/** The uid of a member in the channel.
 */
@property (nonatomic, copy, nonnull) NSString *userId;
/** The channel ID.
 */
@property (nonatomic, copy, nonnull) NSString *channelId;
@end


/** Occurs after the app calls the [joinWithCompletion](joinWithCompletion) method.

 @param errorCode [AgoraRtmJoinChannelBlock](AgoraRtmJoinChannelBlock), which indicates the result of a user joining a channel. See AgoraRtmJoinChannelErrorCode for the error codes.
 */
typedef void (^AgoraRtmJoinChannelBlock)(AgoraRtmJoinChannelErrorCode errorCode);
/** Occurs after the app calls the [leaveWithCompletion](leaveWithCompletion) method.

 @param errorCode [AgoraRtmLeaveChannelBlock](AgoraRtmLeaveChannelBlock), which indicates the result of a user leaving a channel. See AgoraRtmLeaveChannelErrorCode for the error codes.
 */
typedef void (^AgoraRtmLeaveChannelBlock)(AgoraRtmLeaveChannelErrorCode errorCode);
/** Occurs after the app calls the [sendMessage]([sendMessage:completion:]) method.

@param errorCode [AgoraRtmSendChannelMessageBlock](AgoraRtmSendChannelMessageBlock), which indicates the result of sending the channel message. See AgoraRtmSendChannelMessageErrorCode for the error codes.
*/
typedef void (^AgoraRtmSendChannelMessageBlock)(AgoraRtmSendChannelMessageErrorCode errorCode);
/** Occurs after the app calls the [getMembersWithCompletion](getMembersWithCompletion) method.

@param members The member list.
@param errorCode [AgoraRtmGetMembersBlock](AgoraRtmGetMembersBlock), which indicates the result of retrieving the member list of a channel. See AgoraRtmGetMembersErrorCode for the error codes.
*/
typedef void (^AgoraRtmGetMembersBlock)(NSArray< AgoraRtmMember *> * _Nullable members, AgoraRtmGetMembersErrorCode errorCode);

/** The AgoraRtmChannelDelegate protocol enables callback event notifications to your app.

The SDK uses delegate callbacks in the AgoraRtmChannelDelegate protocol to report AgoraRtmChannelDelegate runtime events to the app.
*/
@protocol AgoraRtmChannelDelegate <NSObject>
@optional
/** Occurs when a remote user joins a channel.
 
 **Note:** This callback is disabled when the number of the channel members exceeds 512.

@param channel The channel that the user joins. See AgoraRtmChannel.
@param member The user ID of the user joining the channel.
*/
- (void)channel:(AgoraRtmChannel * _Nonnull)channel memberJoined:(AgoraRtmMember * _Nonnull)member;
/** Occurs when a remote user leaves a channel.
 
 **Note:** This callback is disabled when the number of the channel members exceeds 512.

@param channel The channel that the user leaves. See AgoraRtmChannel.
@param member The user ID of the user leaving the channel.
*/
- (void)channel:(AgoraRtmChannel * _Nonnull)channel memberLeft:(AgoraRtmMember * _Nonnull)member;
/** Occurs when the local user receives a channel message.

@param channel The channel of the user receiving the channel message. See AgoraRtmChannel.
@param message The received channel message.
@param member The user ID of the sender.
*/
- (void)channel:(AgoraRtmChannel * _Nonnull)channel messageReceived:(AgoraRtmMessage * _Nonnull)message fromMember:(AgoraRtmMember * _Nonnull)member;
@end

/** Agora RTM channel methods.
 */
__attribute__((visibility("default"))) @interface AgoraRtmChannel: NSObject

/** An [AgoraRtmKit](AgoraRtmKit) instance. */
@property (atomic, readonly, nonnull) AgoraRtmKit *kit;
/** AgoraRtmChannelDelegate enables Agora RTM channel callback event notifications to your app. */
@property (nonatomic, weak, nullable) id<AgoraRtmChannelDelegate> channelDelegate;

/** Allows a user to join a channel.

- Success:
    - The local user receives the [AgoraRtmJoinChannelBlock](AgoraRtmJoinChannelBlock) callback with the `AgoraRtmJoinChannelErrorOk` state.
    - All remote users receive the [memberJoined]([AgoraRtmChannelDelegate channel:memberJoined:]) callback.
- Failure: The local user receives the [AgoraRtmJoinChannelBlock](AgoraRtmJoinChannelBlock) callback with an error code. See AgoraRtmJoinChannelErrorCode for the error codes.

@param completionBlock [AgoraRtmJoinChannelBlock](AgoraRtmJoinChannelBlock), which indicates the result of joining a channel. See AgoraRtmJoinChannelErrorCode for the error codes.

 */
- (void)joinWithCompletion:(AgoraRtmJoinChannelBlock _Nullable)completionBlock;
/** Allows a user to leave a channel.

- Success:
    - The local user receives the [AgoraRtmLeaveChannelBlock](AgoraRtmLeaveChannelBlock) callback with the `AgoraRtmLeaveChannelErrorOk` state.
    - All remote users receive the [memberLeft]([AgoraRtmChannelDelegate channel:memberLeft:]) callback.
- Failure: The local user receives the [AgoraRtmLeaveChannelBlock](AgoraRtmLeaveChannelBlock) callback with an error code. See AgoraRtmLeaveChannelErrorCode for the error codes.

@param completionBlock [AgoraRtmLeaveChannelBlock](AgoraRtmLeaveChannelBlock), which indicates the result of leaving a channel. See AgoraRtmLeaveChannelErrorCode for the error codes.

 */
- (void)leaveWithCompletion:(AgoraRtmLeaveChannelBlock _Nullable)completionBlock;

/** Allows a user to send a message to all users in the channel.
 
 **Note:** You can send channel messages at a maximum speed of 60 queries per second.

- Success:
    - The local user receives the [AgoraRtmSendChannelMessageBlock](AgoraRtmSendChannelMessageBlock) callback with the `AgoraRtmSendChannelMessageErrorOk` state.
    - All remote users receive the [messageReceived]([AgoraRtmChannelDelegate channel:messageReceived:fromMember:]) callback.
- Failure: The local user receives the [AgoraRtmSendChannelMessageBlock](AgoraRtmSendChannelMessageBlock) callback with an error. See AgoraRtmSendChannelMessageErrorCode for the error codes. 

@param message The message to be sent.

@param completionBlock [AgoraRtmSendChannelMessageBlock](AgoraRtmSendChannelMessageBlock), which indicates the result of sending a channel message. See AgoraRtmSendChannelMessageErrorCode for the error codes.

 */
- (void)sendMessage:(AgoraRtmMessage * _Nonnull)message
         completion:(AgoraRtmSendChannelMessageBlock _Nullable)completionBlock;
/** Retrieves the member list of a channel.

@param completionBlock [AgoraRtmGetMembersBlock](AgoraRtmGetMembersBlock), which indicates the result of getting the member list of the channel. See AgoraRtmGetMembersErrorCode for the error codes.
 */
- (void)getMembersWithCompletion:(AgoraRtmGetMembersBlock _Nullable)completionBlock;
@end
